package sun.awt.windows;

import java.awt.Color;
import java.util.Collections;
import java.util.HashMap;
import java.awt.peer.ComponentPeer;
import java.awt.peer.LightweightPeer;
import java.awt.event.InvocationEvent;
import java.awt.Rectangle;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.InputMethodEvent;
import java.awt.font.TextHitInfo;
import sun.awt.SunToolkit;
import java.text.Annotation;
import java.text.AttributedCharacterIterator;
import java.text.AttributedString;
import java.awt.im.InputMethodHighlight;
import java.awt.event.ComponentEvent;
import java.awt.AWTEvent;
import java.awt.im.InputSubset;
import java.awt.font.TextAttribute;
import java.util.Map;
import java.util.Locale;
import java.awt.Component;
import java.awt.im.spi.InputMethodContext;
import sun.awt.im.InputMethodAdapter;

final class WInputMethod extends InputMethodAdapter
{
    private InputMethodContext inputContext;
    private Component awtFocussedComponent;
    private WComponentPeer awtFocussedComponentPeer;
    private WComponentPeer lastFocussedComponentPeer;
    private boolean isLastFocussedActiveClient;
    private boolean isActive;
    private int context;
    private boolean open;
    private int cmode;
    private Locale currentLocale;
    private boolean statusWindowHidden;
    public static final byte ATTR_INPUT = 0;
    public static final byte ATTR_TARGET_CONVERTED = 1;
    public static final byte ATTR_CONVERTED = 2;
    public static final byte ATTR_TARGET_NOTCONVERTED = 3;
    public static final byte ATTR_INPUT_ERROR = 4;
    public static final int IME_CMODE_ALPHANUMERIC = 0;
    public static final int IME_CMODE_NATIVE = 1;
    public static final int IME_CMODE_KATAKANA = 2;
    public static final int IME_CMODE_LANGUAGE = 3;
    public static final int IME_CMODE_FULLSHAPE = 8;
    public static final int IME_CMODE_HANJACONVERT = 64;
    public static final int IME_CMODE_ROMAN = 16;
    private static final boolean COMMIT_INPUT = true;
    private static final boolean DISCARD_INPUT = false;
    private static Map<TextAttribute, Object>[] highlightStyles;
    
    public WInputMethod() {
        this.awtFocussedComponentPeer = null;
        this.lastFocussedComponentPeer = null;
        this.isLastFocussedActiveClient = false;
        this.statusWindowHidden = false;
        this.context = this.createNativeContext();
        this.cmode = this.getConversionStatus(this.context);
        this.open = this.getOpenStatus(this.context);
        this.currentLocale = getNativeLocale();
        if (this.currentLocale == null) {
            this.currentLocale = Locale.getDefault();
        }
    }
    
    @Override
    protected void finalize() throws Throwable {
        if (this.context != 0) {
            this.destroyNativeContext(this.context);
            this.context = 0;
        }
        super.finalize();
    }
    
    @Override
    public synchronized void setInputMethodContext(InputMethodContext inputContext) {
        this.inputContext = inputContext;
    }
    
    @Override
    public final void dispose() {
    }
    
    @Override
    public Object getControlObject() {
        return null;
    }
    
    @Override
    public boolean setLocale(Locale locale) {
        return this.setLocale(locale, false);
    }
    
    private boolean setLocale(Locale locale, boolean b) {
        Locale[] availableLocalesInternal = WInputMethodDescriptor.getAvailableLocalesInternal();
        int i = 0;
        while (i < availableLocalesInternal.length) {
            Locale locale2 = availableLocalesInternal[i];
            if (locale.equals(locale2) || (locale2.equals(Locale.JAPAN) && locale.equals(Locale.JAPANESE)) || (locale2.equals(Locale.KOREA) && locale.equals(Locale.KOREAN))) {
                if (this.isActive) {
                    setNativeLocale(locale2.toLanguageTag(), b);
                }
                this.currentLocale = locale2;
                return true;
            }
            else {
                ++i;
            }
        }
        return false;
    }
    
    @Override
    public Locale getLocale() {
        if (this.isActive) {
            this.currentLocale = getNativeLocale();
            if (this.currentLocale == null) {
                this.currentLocale = Locale.getDefault();
            }
        }
        return this.currentLocale;
    }
    
    @Override
    public void setCharacterSubsets(Character.Subset[] array) {
        if (array == null) {
            this.setConversionStatus(this.context, this.cmode);
            this.setOpenStatus(this.context, this.open);
            return;
        }
        Character.Subset subset = array[0];
        Locale nativeLocale = getNativeLocale();
        if (nativeLocale == null) {
            return;
        }
        if (nativeLocale.getLanguage().equals(Locale.JAPANESE.getLanguage())) {
            if (subset == Character.UnicodeBlock.BASIC_LATIN || subset == InputSubset.LATIN_DIGITS) {
                this.setOpenStatus(this.context, false);
            }
            else {
                int n3;
                if (subset == Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS || subset == InputSubset.KANJI || subset == Character.UnicodeBlock.HIRAGANA) {
                    n3 = 9;
                }
                else if (subset == Character.UnicodeBlock.KATAKANA) {
                    n3 = 11;
                }
                else if (subset == InputSubset.HALFWIDTH_KATAKANA) {
                    n3 = 3;
                }
                else if (subset == InputSubset.FULLWIDTH_LATIN) {
                    n3 = 8;
                }
                else {
                    return;
                }
                this.setOpenStatus(this.context, true);
                this.setConversionStatus(this.context, n3 | (this.getConversionStatus(this.context) & 0x10));
            }
        }
        else if (nativeLocale.getLanguage().equals(Locale.KOREAN.getLanguage())) {
            if (subset == Character.UnicodeBlock.BASIC_LATIN || subset == InputSubset.LATIN_DIGITS) {
                this.setOpenStatus(this.context, false);
            }
            else {
                int n2;
                if (subset == Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS || subset == InputSubset.HANJA || subset == Character.UnicodeBlock.HANGUL_SYLLABLES || subset == Character.UnicodeBlock.HANGUL_JAMO || subset == Character.UnicodeBlock.HANGUL_COMPATIBILITY_JAMO) {
                    n2 = 1;
                }
                else if (subset == InputSubset.FULLWIDTH_LATIN) {
                    n2 = 8;
                }
                else {
                    return;
                }
                this.setOpenStatus(this.context, true);
                this.setConversionStatus(this.context, n2);
            }
        }
        else if (nativeLocale.getLanguage().equals(Locale.CHINESE.getLanguage())) {
            if (subset == Character.UnicodeBlock.BASIC_LATIN || subset == InputSubset.LATIN_DIGITS) {
                this.setOpenStatus(this.context, false);
            }
            else {
                int n;
                if (subset == Character.UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS || subset == InputSubset.TRADITIONAL_HANZI || subset == InputSubset.SIMPLIFIED_HANZI) {
                    n = 1;
                }
                else if (subset == InputSubset.FULLWIDTH_LATIN) {
                    n = 8;
                }
                else {
                    return;
                }
                this.setOpenStatus(this.context, true);
                this.setConversionStatus(this.context, n);
            }
        }
        return;
    }
    
    @Override
    public void dispatchEvent(AWTEvent awtEvent) {
        if ((awtEvent instanceof ComponentEvent)) {
            Component component = ((ComponentEvent)awtEvent).getComponent();
            if (component == this.awtFocussedComponent) {
                if (this.awtFocussedComponentPeer == null || this.awtFocussedComponentPeer.isDisposed()) {
                    this.awtFocussedComponentPeer = this.getNearestNativePeer(component);
                }
                if (this.awtFocussedComponentPeer != null) {
                    this.handleNativeIMEEvent(this.awtFocussedComponentPeer, awtEvent);
                }
            }
        }
    }
    
    @Override
    public void activate() {
        int haveActiveClient = this.haveActiveClient();
        if (this.lastFocussedComponentPeer != this.awtFocussedComponentPeer || (this.isLastFocussedActiveClient ? 1 : 0) != haveActiveClient) {
            if (this.lastFocussedComponentPeer != null) {
                this.disableNativeIME(this.lastFocussedComponentPeer);
            }
            if (this.awtFocussedComponentPeer != null) {
                this.enableNativeIME(this.awtFocussedComponentPeer, this.context, (boolean)(haveActiveClient == 0));
            }
            this.lastFocussedComponentPeer = this.awtFocussedComponentPeer;
            this.isLastFocussedActiveClient = haveActiveClient;
        }
        this.isActive = true;
        if (this.currentLocale != null) {
            this.setLocale(this.currentLocale, true);
        }
        if (this.statusWindowHidden) {
            this.setStatusWindowVisible(this.awtFocussedComponentPeer, true);
            this.statusWindowHidden = false;
        }
    }
    
    @Override
    public void deactivate(boolean b) {
        this.getLocale();
        if (this.awtFocussedComponentPeer != null) {
            this.lastFocussedComponentPeer = this.awtFocussedComponentPeer;
            this.isLastFocussedActiveClient = this.haveActiveClient();
        }
        this.isActive = false;
    }
    
    @Override
    public void disableInputMethod() {
        if (this.lastFocussedComponentPeer != null) {
            this.disableNativeIME(this.lastFocussedComponentPeer);
            this.lastFocussedComponentPeer = null;
            this.isLastFocussedActiveClient = false;
        }
    }
    
    @Override
    public String getNativeInputMethodInfo() {
        return this.getNativeIMMDescription();
    }
    
    @Override
    protected void stopListening() {
        this.disableInputMethod();
    }
    
    @Override
    protected void setAWTFocussedComponent(Component awtFocussedComponent) {
        if (awtFocussedComponent == null) {
            return;
        }
        WComponentPeer nearestNativePeer = this.getNearestNativePeer(awtFocussedComponent);
        if (this.isActive) {
            if (this.awtFocussedComponentPeer != null) {
                this.disableNativeIME(this.awtFocussedComponentPeer);
            }
            if (nearestNativePeer != null) {
                this.enableNativeIME(nearestNativePeer, this.context, !this.haveActiveClient());
            }
        }
        this.awtFocussedComponent = awtFocussedComponent;
        this.awtFocussedComponentPeer = nearestNativePeer;
        return;
    }
    
    @Override
    public void hideWindows() {
        if (this.awtFocussedComponentPeer != null) {
            this.setStatusWindowVisible(this.awtFocussedComponentPeer, false);
            this.statusWindowHidden = true;
        }
    }
    
    @Override
    public void removeNotify() {
        this.endCompositionNative(this.context, false);
        this.awtFocussedComponent = null;
        this.awtFocussedComponentPeer = null;
    }
    
    static Map<TextAttribute, ?> mapInputMethodHighlight(InputMethodHighlight inputMethodHighlight) {
        int state = inputMethodHighlight.getState();
        int n;
        if (state == 0) {
            n = 0;
        }
        else if (state == 1) {
            n = 2;
        }
        else {
            return null;
        }
        if (inputMethodHighlight.isSelected()) {
            ++n;
        }
        return WInputMethod.highlightStyles[n];
    }
    
    @Override
    protected boolean supportsBelowTheSpot() {
        return true;
    }
    
    @Override
    public void endComposition() {
        this.endCompositionNative(this.context, this.haveActiveClient());
    }
    
    @Override
    public void setCompositionEnabled(boolean b) {
        this.setOpenStatus(this.context, b);
    }
    
    @Override
    public boolean isCompositionEnabled() {
        return this.getOpenStatus(this.context);
    }
    
    public void sendInputMethodEvent(int id, long when, String text, int[] array, String[] array2, int[] array3, byte[] array4, int committedCharacterCount, int charIndex, int charIndex2) {
        AttributedCharacterIterator iterator = null;
        if (text != null) {
            AttributedString attributedString = new AttributedString(text);
            attributedString.addAttribute(AttributedCharacterIterator.Attribute.LANGUAGE, Locale.getDefault(), 0, text.length());
            if (array != null && array2 != null && array2.length != 0 && array.length == array2.length + 1 && array[0] == 0 && array[array2.length] <= text.length()) {
                for (int i = 0; i < array.length - 1; ++i) {
                    attributedString.addAttribute(AttributedCharacterIterator.Attribute.INPUT_METHOD_SEGMENT, new Annotation(null), array[i], array[i + 1]);
                    attributedString.addAttribute(AttributedCharacterIterator.Attribute.READING, new Annotation(array2[i]), array[i], array[i + 1]);
                }
            }
            else {
                attributedString.addAttribute(AttributedCharacterIterator.Attribute.INPUT_METHOD_SEGMENT, new Annotation(null), 0, text.length());
                attributedString.addAttribute(AttributedCharacterIterator.Attribute.READING, new Annotation(""), 0, text.length());
            }
            if (array3 != null && array4 != null && array4.length != 0 && array3.length == array4.length + 1 && array3[0] == 0 && array3[array4.length] == text.length()) {
                for (int j = 0; j < array3.length - 1; ++j) {
                    InputMethodHighlight value;
                    switch (array4[j]) {
                        case 1: {
                            value = InputMethodHighlight.SELECTED_CONVERTED_TEXT_HIGHLIGHT;
                            break;
                        }
                        case 2: {
                            value = InputMethodHighlight.UNSELECTED_CONVERTED_TEXT_HIGHLIGHT;
                            break;
                        }
                        case 3: {
                            value = InputMethodHighlight.SELECTED_RAW_TEXT_HIGHLIGHT;
                            break;
                        }
                        default: {
                            value = InputMethodHighlight.UNSELECTED_RAW_TEXT_HIGHLIGHT;
                            break;
                        }
                    }
                    attributedString.addAttribute(TextAttribute.INPUT_METHOD_HIGHLIGHT, value, array3[j], array3[j + 1]);
                }
            }
            else {
                attributedString.addAttribute(TextAttribute.INPUT_METHOD_HIGHLIGHT, InputMethodHighlight.UNSELECTED_CONVERTED_TEXT_HIGHLIGHT, 0, text.length());
            }
            iterator = attributedString.getIterator();
        }
        Component clientComponent = this.getClientComponent();
        if (clientComponent == null) {
            return;
        }
        SunToolkit.postEvent(SunToolkit.targetToAppContext(clientComponent), new InputMethodEvent(clientComponent, id, when, iterator, committedCharacterCount, TextHitInfo.leading(charIndex), TextHitInfo.leading(charIndex2)));
        return;
    }
    
    public void inquireCandidatePosition() {
        Component clientComponent = this.getClientComponent();
        if (clientComponent == null) {
            return;
        }
        SunToolkit.postEvent(SunToolkit.targetToAppContext(clientComponent), new InvocationEvent(clientComponent, new Runnable() {
            @Override
            public void run() {
                int n = 0;
                int n2 = 0;
                Component access$000 = WInputMethod.access$000(WInputMethod.this);
                if (access$000 != null) {
                    if (!access$000.isShowing()) {
                        return;
                    }
                    if (WInputMethod.access$100(WInputMethod.this)) {
                        Rectangle textLocation = WInputMethod.this.inputContext.getTextLocation(TextHitInfo.leading(0));
                        n = textLocation.x;
                        n2 = textLocation.y + textLocation.height;
                    }
                    else {
                        Point locationOnScreen = access$000.getLocationOnScreen();
                        Dimension size = access$000.getSize();
                        n = locationOnScreen.x;
                        n2 = locationOnScreen.y + size.height;
                    }
                }
                WInputMethod.access$400(WInputMethod.this, WInputMethod.this.awtFocussedComponentPeer, n, n2);
            }
        }));
        return;
    }
    
    private WComponentPeer getNearestNativePeer(Component parent) {
        if (parent == null) {
            return null;
        }
        ComponentPeer componentPeer = parent.getPeer();
        if (componentPeer == null) {
            return null;
        }
        while (componentPeer instanceof LightweightPeer) {
            parent = parent.getParent();
            if (parent == null) {
                return null;
            }
            componentPeer = parent.getPeer();
            if (componentPeer != null) {
                continue;
            }
            return null;
        }
        if ((componentPeer instanceof WComponentPeer)) {
            return (WComponentPeer)componentPeer;
        }
        return null;
    }
    
    private native int createNativeContext();
    
    private native void destroyNativeContext(int p0);
    
    private native void enableNativeIME(WComponentPeer p0, int p1, boolean p2);
    
    private native void disableNativeIME(WComponentPeer p0);
    
    private native void handleNativeIMEEvent(WComponentPeer p0, AWTEvent p1);
    
    private native void endCompositionNative(int p0, boolean p1);
    
    private native void setConversionStatus(int p0, int p1);
    
    private native int getConversionStatus(int p0);
    
    private native void setOpenStatus(int p0, boolean p1);
    
    private native boolean getOpenStatus(int p0);
    
    private native void setStatusWindowVisible(WComponentPeer p0, boolean p1);
    
    private native String getNativeIMMDescription();
    
    static native Locale getNativeLocale();
    
    static native boolean setNativeLocale(String p0, boolean p1);
    
    private native void openCandidateWindow(WComponentPeer p0, int p1, int p2);
    
    static Component access$000(WInputMethod wInputMethod) {
        return wInputMethod.getClientComponent();
    }
    
    static boolean access$100(WInputMethod wInputMethod) {
        return wInputMethod.haveActiveClient();
    }
    
    static void access$400(WInputMethod wInputMethod, WComponentPeer wComponentPeer, int n, int n2) {
        wInputMethod.openCandidateWindow(wComponentPeer, n, n2);
    }
    
    static {
        Map[] highlightStyles = new Map[4];
        HashMap m = new HashMap(1);
        m.put(TextAttribute.INPUT_METHOD_UNDERLINE, TextAttribute.UNDERLINE_LOW_DOTTED);
        highlightStyles[0] = Collections.unmodifiableMap(m);
        HashMap i = new HashMap(1);
        i.put(TextAttribute.INPUT_METHOD_UNDERLINE, TextAttribute.UNDERLINE_LOW_GRAY);
        highlightStyles[1] = Collections.unmodifiableMap(i);
        HashMap j = new HashMap(1);
        j.put(TextAttribute.INPUT_METHOD_UNDERLINE, TextAttribute.UNDERLINE_LOW_DOTTED);
        highlightStyles[2] = Collections.unmodifiableMap(j);
        HashMap k = new HashMap(4);
        k.put(TextAttribute.FOREGROUND, new Color(0, 0, 128));
        k.put(TextAttribute.BACKGROUND, Color.white);
        k.put(TextAttribute.SWAP_COLORS, TextAttribute.SWAP_COLORS_ON);
        k.put(TextAttribute.INPUT_METHOD_UNDERLINE, TextAttribute.UNDERLINE_LOW_ONE_PIXEL);
        highlightStyles[3] = Collections.unmodifiableMap(k);
        WInputMethod.highlightStyles = highlightStyles;
    }
}
